# -*- coding: iso-8859-15 -*-
#appModules/wlmail.py modifie par Daniel Poiraud, AVH16. version 3.3 du 22/04/2022.
#A part of NonVisual Desktop Access (NVDA)
#from __future__ import unicode_literals
import winUser
from winUser import *
import controlTypes
if not hasattr(controlTypes, "Role"):
	setattr(controlTypes, "Role", type('Enum', (), dict(
	[(x.split("ROLE_")[1], getattr(controlTypes, x)) for x in dir(controlTypes) if x.startswith("ROLE_")])))
	setattr(controlTypes, "State", type('Enum', (), dict(
	[(x.split("STATE_")[1], getattr(controlTypes, x)) for x in dir(controlTypes) if x.startswith("STATE_")])))
	setattr(controlTypes, "role", type("role", (), {"_roleLabels": controlTypes.roleLabels}))
	
import textInfos
import api
import nvdaBuiltin.appModules.wlmail
import speech
from scriptHandler import getLastScriptRepeatCount
from ui import message
import ctypes
from ctypes import windll
from ctypes import * 
from ctypes.wintypes import * 
from keyboardHandler import KeyboardInputGesture
from NVDAObjects import NVDAObject
from NVDAObjects.window import Window
from NVDAObjects.IAccessible import sysListView32, IAccessible, getNVDAObjectFromEvent, getNVDAObjectFromPoint
from NVDAObjects.IAccessible.MSHTML import MSHTML
from time import sleep
import wx
import windowUtils
import displayModel
import watchdog
import config

import browseMode
import NVDAObjects
import treeInterceptorHandler
from NVDAObjects.behaviors import EditableTextWithAutoSelectDetection, EditableTextWithSuggestions

try :
	from . import divers
	from . import wlmfenredaction
	from . import wlmentetes
	from .wlmfenredaction import *
	from .wlmentetes import *
	from .divers import *
except :
	import wlmfenredaction
	import wlmentetes
	import divers
	from wlmfenredaction import *
	from wlmentetes import *
	from  divers import *	
	# pass
from UIAHandler import handler, IUIAutomationLegacyIAccessiblePattern
clientObject = handler.clientObject
user32 = windll.user32
UIA_BoundingRectanglePropertyId, UIA_ControlTypePropertyId, UIA_NamePropertyId, UIA_ButtonControlTypeId = 30001, 30003, 30005, 50000

def getSpeechMode():
	try:
		return speech.getState().speechMode
	except AttributeError:
		return speech.speechMode
def setSpeechMode(mode):
	try:
		speech.setSpeechMode(mode)
	except AttributeError:
		speech.speechMode = mode

def setSpeechMode_off():
	try:
		speech.setSpeechMode(speech.SpeechMode.off)
	except AttributeError:
		speech.speechMode = speech.speechMode_off
		
class WlmDocumentTreeInterceptor (browseMode.BrowseModeDocumentTreeInterceptor):
	pass
			
class WlmDocument (EditableTextWithAutoSelectDetection, EditableTextWithSuggestions):

	treeInterceptorClass =WlmDocumentTreeInterceptor # On dfinit la class NotepadDocumentTreeInterceptor en tant qu'attribut treeInterceptorClass de notre objet NVDA personnalis.
	shouldCreateTreeInterceptor = False  		

	def script_goToHeading (self, gesture):
		u"""
		Script permettant d'aller vers le titre prcdent ou suivant, que l'on soit en mode navigation ou pas.
		Contrle + R va vers le titre suivant.
		Contrle + Shift + R va vers le titre prcdent.
		"""
		focus = api.getFocusObject() # On enregistre la position du focus.
		if hasattr (self, "treeInterceptor") and self.treeInterceptor:
			# Nous sommes dj en mode navigation.
			obj = self.treeInterceptor
		else:
			# Nous forons le passage en mode navigation.
			ti = treeInterceptorHandler.update(self, force=True)
			if focus in ti:
				focus.treeInterceptor = ti
				obj = focus.treeInterceptor
		if "shift" in gesture.modifierNames: # Shift est prsent dans les touches d'appel du script, nous revenons en arrire.
			obj.script_previousHeading(gesture)
		else: # Nous avanons vers le bas.
			obj.script_nextHeading(gesture)

	__gestures = {
		"kb:control+r":"goToHeading",
		"kb:control+shift+r":"goToHeading",
	}

class AppModule(nvdaBuiltin.appModules.wlmail.AppModule):
	
	def __init__(self, *args, **kwargs):
		# Appel du constructeur de la classe parente
		super(AppModule, self).__init__(*args, **kwargs)
		self.oldReportTooltips = config.conf["presentation"]["reportTooltips"]
		self.oldreportTables=config.conf["documentFormatting"]["reportTableHeaders"]
		#self.maximiser() 	#Pb avec les rubans 		
	
	def event_foreground(self, obj, nextHandler):
		if not self.dansFenetreRedaction() :
			config.conf["presentation"]["reportTooltips"] = self.oldReportTooltips
		nextHandler	
				
	def terminate(self):
		print ("Terminate")
		config.conf["presentation"]["reportTooltips"] = self.oldReportTooltips
		config.conf["documentFormatting"]["reportTableHeaders"] = self.oldreportTables
		super(AppModule, self).terminate()
		
	def chooseNVDAObjectOverlayClasses(self, obj, clsList):
		super(AppModule, self).chooseNVDAObjectOverlayClasses(obj, clsList)
		if obj.role ==controlTypes.Role.EDITABLETEXT :
			clsList.insert(0,WlmDocument)
		if api.getFocusObject().windowClassName=="SysListView32" and obj.role==controlTypes.Role.LISTITEM :
			config.conf["documentFormatting"]["reportTableHeaders"] = False
		if (obj.windowClassName=="WLXDUI" or obj.windowClassName=="DirectUIHWND") and obj.role==controlTypes.Role.EDITABLETEXT and (obj.name==u" :" or obj.name=="Cc :"  or obj.name=="Cci :"):
			config.conf["presentation"]["reportTooltips"] = True
			#config.conf["documentFormatting"]["reportTables"] = False
			clsList.insert(0,wlmfenredaction.PourAutoCompletion)				
		try :
			if (winUser.getWindowText(winUser.getForegroundWindow())==u"Vrifier l'orthographe") or (winUser.getWindowText(winUser.getForegroundWindow())==u"Orthographe") :
				clsList.insert(0,wlmfenredaction.PourCorrection)	
		except :
			pass
		try :
			if (api.getForegroundObject().windowClassName=="MailContactPickerDialog_Class") :
				clsList.insert(0,wlmfenredaction.PourBDDSelectionContacts)	 
		except :
			pass					 

	def dansFenetreRedaction(self):
		if not getClassName(user32.GetForegroundWindow())=="ATH_Note" : return
		nomBoutonTest1 = "Envoyer ce message maintenant" 
		nomBoutonTest2 = "Format"		
		oCondition1  =clientObject.CreatePropertyCondition(UIA_ControlTypePropertyId,UIA_ButtonControlTypeId) 
		oCondition2  =clientObject.CreatePropertyCondition(UIA_NamePropertyId, nomBoutonTest1)
		oCondition3  =clientObject.CreatePropertyCondition(UIA_NamePropertyId, nomBoutonTest2) 
		oCondition4  =clientObject.CreateOrCondition(oCondition2,oCondition3) 
		oCondition  =clientObject.CreateAndCondition(oCondition1,oCondition4) 
		if not clientObject.ElementFromHandle(user32.GetForegroundWindow()).FindFirst(4,oCondition):
				return 0
		else : return 1

	def maximiser(self):
		hdMain=user32.GetForegroundWindow()
		if winUser.getClassName(hdMain)=="Outlook Express Browser Class" or winUser.getClassName(hdMain)=="ATH_Note":
			sendMessage (hdMain,0x112, 0xF030,0)							
		
	def script_hotKeyHelp(self,gesture):
		divers.HotKeyHelp.run()

	def script_ouvrirBDDSelectionContacts(self,gesture):
		if not (winUser.getClassName(winUser.getForegroundWindow())=="ATH_Note") :
			message(u"Vous n'tes pas dans la fentre de rdaction")
			return	
		try :
			oCondition1  =clientObject.CreatePropertyCondition(UIA_ControlTypePropertyId,UIA_ButtonControlTypeId)
			oCondition2  =clientObject.CreatePropertyCondition(UIA_NamePropertyId,u"Slectionner les destinataires")
			oCondition  =clientObject.CreateAndCondition(oCondition1,oCondition2)
			element = clientObject.ElementFromHandle(user32.GetForegroundWindow()).FindFirst(4,oCondition)
		except : 
			message(u"Boite de dialogue non trouve")
			return
		(x,y,l,h) =element.GetCurrentPropertyValue(UIA_BoundingRectanglePropertyId)
		winUser.setCursorPos(int(x)+10,int(y)+8)
		mouse_event(MOUSEEVENTF_LEFTDOWN,0,0,None,None)
		mouse_event(MOUSEEVENTF_LEFTUP,0,0,None,None)	
		sleep(0.5)
		winUser.setCursorPos(3,int(y)+8)
	
	def script_allerListeMessages(self,gesture):
		o = api.getFocusObject() 
		if o.windowClassName=="SysListView32" :
			message(u"Vous tes dans la liste des messages.")
			message(o.name)
			return	
		if not (winUser.getClassName(user32.GetForegroundWindow())=="Outlook Express Browser Class") : return
		hdmain, nomClass =user32.GetForegroundWindow(),""	
		o = api.getForegroundObject()
		try :
			if "UIRibbon" in o.firstChild.name :		
				nomClass ="WLXDUI"
		except : 	
				nomClass ="DirectUIHWND"
		try :
			hd1 = windowUtils.findDescendantWindow(hdmain,visible= True,className="Outlook Express Message View")
			hd2 = windowUtils.findDescendantWindow(hd1,visible= True,className=nomClass)
			hd3 = windowUtils.findDescendantWindow(hd2,visible= True,className="OE Live Message List Host")		
			hd4 = windowUtils.findDescendantWindow(hd3,visible= True,className="SysListView32")	
			if hd4 : 
				o = getNVDAObjectFromEvent(hd4,-4,0) 
				nb = o.childCount
				if nb >400 : message ("Patientez")
				oldSpeechMode = getSpeechMode()
				setSpeechMode_off()
				sleep(0.1)
				# for i in list( range (0,nb-1)):  
				for i in range (0,nb-1):  
					if controlTypes.State.SELECTED in o.children[i].states :   # 4
						o.children[i].setFocus()
						break 
					if controlTypes.State.FOCUSED in o.children[i].states : # 2
						o.children[i].setFocus()
						break	
					##if controlTypes.STATE_INVISIBLE in o.children[i].states  or controlTypes.STATE_UNAVAILABLE in o.children[i].states:				
					if i ==nb-1 and not (controlTypes.State.FOCUSED in o.children[nb-1].states or controlTypes.State.SELECTED in o.children[nb-1].states):
						o.children[0].setFocus()
				api.processPendingEvents()
				sleep(0.1)
				setSpeechMode(oldSpeechMode)		
				message(o.children[i].name)
		except :
			message ("Liste des messages non atteinte")  

	def script_allerZoneRedaction (self,gesture): 
		if not self.dansFenetreRedaction() : 
			message(u"Vous n'tes pas dans la fentre de rdaction")
			return
		pourAllerContenu = wlmfenredaction.PourAutoCompletion()	
		pourAllerContenu.allerContenu()
		
	def script_enTeteDe(self,gesture):
		pourEnTetes = wlmentetes.PourEnTetes()
		pourEnTetes.enTete(1)
		
	def script_enTeteDate(self,gesture):
		if self.dansFenetreRedaction() :
			message(u"Pas de champ Date dans cette fentre.")
			return
		pourEnTetes = wlmentetes.PourEnTetes()
		pourEnTetes.enTete(2)
		
	def script_enTeteA(self,gesture):
		pourEnTetes = wlmentetes.PourEnTetes()
		pourEnTetes.enTete(3)
		
	def script_enTeteCC(self,gesture):
		pourEnTetes = wlmentetes.PourEnTetes()
		pourEnTetes.enTete(4)
		
	def script_enTeteCCI(self,gesture):
		pourEnTetes = wlmentetes.PourEnTetes()
		pourEnTetes.enTete(5)	
		
	def script_enTeteSujet(self,gesture):
		pourEnTetes = wlmentetes.PourEnTetes()
		pourEnTetes.enTete(6)		
	
	def script_annoncePieceJointe(self,gesture):
		pourEnTetes = wlmentetes.PourEnTetes()
		pourEnTetes.enTete(7)	
	
	def script_lireContenuMessage(self,gesture):
		pourContenu = wlmentetes.PourEnTetes()
		pourContenu.lireContenu()	
			
	def script_listeDossiersArbo(self,gesture):
		divers.ListeDossiers.run()	
						
	def script_test(self,gesture):	
		message("pour test ") 
		import gui
		if gui.messageBox( u"Le dossier d'arborescence slectionn doit contenir trs peu de messages. Vous continuez?",u"Problme avec Windows Live Mail", wx.YES|wx.NO | wx.CANCEL) !=wx.YES: return 
	__gestures={
"kb:NVDA+Control+h":"hotKeyHelp",
"kb:nvda+K":"ouvrirBDDSelectionContacts", 
"kb:control+L":"allerListeMessages", 
"kb:Alt+1":"enTeteDe",
"kb:Alt+2":"enTeteDate",
"kb:Alt+3":"enTeteA",
"kb:Alt+4":"enTeteCC",
"kb:Alt+5":"enTeteCCI",
"kb:Alt+6":"enTeteSujet",
"kb:Alt+7":"annoncePieceJointe",
"kb:Alt+8":"allerZoneRedaction",
"kb:Windows+Control+downArrow":"lireContenuMessage", 
"kb:control+H":"listeDossiersArbo", 
"kb:control+shift+Y":"test", 
}			