# -*- coding: iso-8859-15 -*-
#A part of NonVisual Desktop Access (NVDA)
# modifie par Daniel Poiraud, AVH16. version 3.3 du 22/04/2022.
import winUser
from winUser import *
import textInfos
import api
import appModuleHandler
import speech
from ui import message
import ctypes
from ctypes import windll
from ctypes import * 
from ctypes.wintypes import * 
from keyboardHandler import KeyboardInputGesture
#from NVDAObjects import NVDAObject
#from NVDAObjects.window import Window
#from NVDAObjects.IAccessible import sysListView32, IAccessible, getNVDAObjectFromEvent, getNVDAObjectFromPoint
#from NVDAObjects.IAccessible.MSHTML import MSHTML
from time import sleep
import wx
import windowUtils
import globalVars
#import displayModel
import watchdog
from NVDAObjects.behaviors import _FakeTableCell
user32 = windll.user32
import UIAHandler
from UIAHandler import handler
clientObject = handler.clientObject
cO=clientObject
CPC=clientObject.CreatePropertyCondition
avecNonLus=False 


def getSpeechMode():
	try:
		return speech.getState().speechMode
	except AttributeError:
		return speech.speechMode
def setSpeechMode(mode):
	try:
		speech.setSpeechMode(mode)
	except AttributeError:
		speech.speechMode = mode

def setSpeechMode_off():
	try:
		speech.setSpeechMode(speech.SpeechMode.off)
	except AttributeError:
		speech.speechMode = speech.speechMode_off

def sansVoix() :
	oldSpeechMode = getSpeechMode()
	setSpeechMode_off()	
	api.processPendingEvents()
	sleep(0.1)
	setSpeechMode(oldSpeechMode)
	
def versionWLM():  
	if appModuleHandler.getAppNameFromProcessID(api.getFocusObject().processID,True) =="wlmail.exe" :
		version = globalVars.foregroundObject.appModule.productVersion.split(".")[0]
		return int(version) 
	else : return 0	
	
def PutWindowOnForeground(hwnd, sleepNb, sleepTime):
	try:
		setForegroundWindow(hwnd)
	except:
		pass
	for i in [sleepTime]*(sleepNb-1):
		sleep(i)
		if winUser.getForegroundWindow() == hwnd:
			return True
	# last chance
	KeyboardInputGesture.fromName("alt+Tab").send()
	sleep(sleepTime)
	if winUser.getForegroundWindow() == hwnd:
		return true
	return False
	
class ListeDossiers(wx.Dialog):
	global avecNonLus, oDeb
	
	def __init__(self, parent,ID, title, closeIfInactive): 
		super(ListeDossiers, self).__init__(parent, ID,title) 
		self.closeIfInactive = closeIfInactive
		self.destroyed = False
		self.getlisteObjets(avecNonLus)
		self.objectTypeHasChanged = True
		self.doGui()

	def doGui(self):
		panel = self
		panelSizer = wx.GridBagSizer (2,4) 
		self.taggedObjectsCB = wx.CheckBox(panel, wx.ID_ANY, label =u"&Afficher seulement les dossiers avec messages non lus")
		self.taggedObjectsCB.SetValue(False) 
		panelSizer.Add(self.taggedObjectsCB, pos=(0,0), span= (1,2), flag=wx.EXPAND|wx.ALL,border=10)
		#the list boxes
		self.listeObjets = wx.ListBox(panel, wx.ID_ANY,  style = wx.LB_SINGLE, size = (300, 450)) 
		if len(self.getlisteObjets(avecNonLus))>0:  
			self.listeObjets.SetItems([obj[0] for obj in self.getlisteObjets(avecNonLus)])
			self.listeObjets.Select(0) 
		panelSizer.Add(self.listeObjets, pos=(1,0), span=(4,1), flag=wx.EXPAND|wx.LEFT|wx.BOTTOM, border=10)
		# the buttons
		self.rightClickButton=wx.Button(panel, wx.NewId(), label =("Simple clic &droit"), size = (120, 30))
		self.leftClickButton=wx.Button(panel, wx.NewId(), label =("Simple clic &gauche"), size = (120, 30))
		self.leftClickButton.SetDefault()		
		self.cancelButton =wx.Button(panel,wx.ID_CANCEL, size = (120, 30))
		panelSizer.Add(self.rightClickButton,(2,1),flag=wx.ALL, border = 10)  
		panelSizer.Add(self.leftClickButton,(3,1),flag=wx.ALL, border = 10)
		panelSizer.Add(self.cancelButton,(4,1),flag = wx.ALL, border= 10)
		panel.SetSizerAndFit(panelSizer)
		# the events
		self.taggedObjectsCB.Bind(wx.EVT_CHECKBOX,self.onCaseChange)
		self.listeObjets.Bind(wx.EVT_KEY_DOWN, self.onKeydown)
		self.leftClickButton.Bind(wx.EVT_BUTTON, self.onLeftMouseButton) 
		self.leftClickButton.Bind(wx.EVT_KEY_DOWN, self.onKeydownButton)  	
		self.rightClickButton.Bind(wx.EVT_KEY_DOWN, self.onKeydownButton) 		
		self.rightClickButton.Bind(wx.EVT_BUTTON, self.onRightMouseButton)
		self.cancelButton.Bind(wx.EVT_BUTTON,self.onCancelButton)
		self.cancelButton.Bind(wx.EVT_KEY_DOWN, self.onKeydownButton)
		self.Bind(wx.EVT_ACTIVATE, self.onActivate)
		self.Bind(wx.EVT_CLOSE, self.onClose)
		self.listeObjets.SetFocus()		

	def getPositionXY (self):
		index=self.listeObjets.GetSelection() 
		locat=self.getlisteObjets(avecNonLus)[index][1] 
		xyItem=(int(locat[0])+int(locat[2]/2),int(locat[1])+int(locat[3]/2))
		return xyItem  

	def onClose(self, evt):
		self.Destroy()
		self.destroyed = True  
		evt.Skip()
		
	def onCheckTaggedObjectsCheckBox(self, evt):
		self.onElementTypeChange(evt)
		
	def getlisteObjets(self,avecNonLus):
		global oDeb
		locat=None 
		name=""
		l=[] 
		if versionWLM() ==16:
			try:  			# pour WLM 2012 a l'ouverture
				obj = api.getFocusObject()
				if  obj.role==3 and not winUser.isWindowVisible(obj.windowHandle):
					ui.message(u"La fentre est inactive. Frappez une touche de dplacement.")
					return
			except:	
				return	
		try :
			hd=oDeb
		except :
			#message ("pb debut")
			return False
		try :
			children=hd.children
			for i in range(1,len(children)):
				if   (avecNonLus) :
					if "Pas de messages non lus" in children[i].value :
						continue
				name =children[i].name
				locat = children[i].location
				l.append((name,locat))
		except:
			#message ("pb liste")
			pass
		return l 

	def onCaseChange(self, evt): 
		global avecNonLus
		avecNonLus = self.taggedObjectsCB.GetValue()
		self.listeObjets.Clear()
		self.listeObjets.SetItems([obj[0] for obj in self.getlisteObjets(avecNonLus) ]) 
		self.objectTypeHasChanged = True
		sleep(0.1)
		if self.listeObjets.GetCount() == 0: message ("Aucun dossier avec non lu")
		sleep(0.1)
		KeyboardInputGesture.fromName("Tab").send()

		
	def onKeydownButton(self, evt):
		keyCode= evt.GetKeyCode()
		if keyCode == wx.WXK_ESCAPE:
			self.Close()
			return
		evt.Skip()

	def onKeydown(self, event):
		keyCode= event.GetKeyCode()
		if keyCode == wx.WXK_ESCAPE:
			self.Close()
			return
		event.Skip()
		
	def onCancelButton(self, evt):
		self.Close()
		evt.Skip()
		
	def onActivate(self, evt):
		# beep(900,60)
		if not self.destroyed and self.closeIfInactive and not evt.GetActive():  
			self.Close()
		evt.Skip()

	def onLeftMouseButton(self,event):
		(x,y)=self.getPositionXY()
		self.Close()
		oldSpeechMode = getSpeechMode()
		setSpeechMode_off()
		winUser.setCursorPos(x,y)
		mouse_event(MOUSEEVENTF_LEFTDOWN,0,0,None,None)
		sleep(0.1)
		mouse_event(MOUSEEVENTF_LEFTUP,0,0,None,None)
		api.processPendingEvents()
		sleep(0.1)
		setSpeechMode(oldSpeechMode)
	
	def onRightMouseButton(self,event):
		(x,y)=self.getPositionXY()
		self.Close()
		oldSpeechMode = getSpeechMode()
		setSpeechMode_off()	
		winUser.setCursorPos(x,y)
		mouse_event(MOUSEEVENTF_RIGHTDOWN,0,0,None,None)
		sleep(0.01)
		mouse_event(MOUSEEVENTF_RIGHTUP,0,0,None,None)		
		api.processPendingEvents()
		sleep(0.1)
		setSpeechMode(oldSpeechMode)

	def run(cls): 
		global oDeb  #volet arborescence des dossiers
		oDeb=api.getForegroundObject().simpleFirstChild.simpleNext.simpleFirstChild.firstChild if  versionWLM()==16 else api.getForegroundObject().simpleFirstChild.simpleFirstChild.next.next.next.firstChild
		bdd = cls(None,wx.ID_ANY,(u"Liste des dossiers dans l'arborescence"), True)
		bdd.CenterOnScreen()
		bdd.Show(True)
		PutWindowOnForeground(bdd.GetHandle(), 4, 0.1) 
	run = classmethod(run)
			
class HotKeyHelp(wx.Dialog):
	def __init__(self, parent, ID, title, closeIfInactive): 
		super(HotKeyHelp, self).__init__(parent, ID,title )
		self.closeIfInactive = closeIfInactive
		self.destroyed = False
		self.doGui()

	def doGui(self):
		mainSizer = wx.BoxSizer(wx.HORIZONTAL)
		box = wx.StaticBox(self,-1,"")
		taskSizer = wx.StaticBoxSizer(box,wx.HORIZONTAL)
		liste = wx.TextCtrl(self,-1,style =wx.TE_MULTILINE|wx.TE_READONLY,size=(560,610)) # 480 430
		# liste des commandes
		comm = u"	Quelques commandes initiales.\r\r"
		comm =comm + u"Se dplacer dans la fentre : F6 et Tab.\r"
		comm =comm + u"Slectionner la boite de rception du compte principal : Ctrl+I.\r"
		comm =comm + u"Synchroniser les comptes de messagerie : F5.\r"
		comm =comm + u"Atteindre le message non lu suivant : Ctrl+U.\r"
		comm =comm + u"Afficher ou non les en-ttes Cc et CCi : Alt+B.\r"
		comm =comm + u"Vrifier une adresse : Ctrl+K.\r"
		comm =comm + u"Vrifier l'orthographe pour un nouveau message : F7.\r"
		comm =comm + u"Accder  un dossier : Ctrl+Y.\r"	
		comm =comm + u"Accder  un dossier contenant des messages non lus : Ctrl+J.\r\r"	
		comm =comm + u"	Commandes pour les adresses des messages  envoyer.\r\r"
		comm =comm + u"Commencer l'utilisation des adresses autocompltes : Ctrl+Maj+Origine ou  Ctrl+Maj+Fin ou Ctrl+Maj+Flche droite\r"
		comm =comm + u"Aller  l'adresse autocomplte suivante : Ctrl+Flche bas.\r"
		comm =comm + u"Aller  l'adresse autocomplte prcdente: Ctrl+Flche haut.\r"
		comm =comm + u"Valider l'adresse autocomplte slectionne: Ctrl+Flche droite.\r"	
		comm =comm + u"Ouvrir la boite de dialogue de slection des contacts : NVDA+K.\r"
		comm =comm + u"Aller dans la zone de recherche de la boite de dialogue de slection des contacts : Alt+K.\r\r"
		comm =comm + u"	Commandes pour les en-ttes.\r\r"	
		comm =comm + u"Lire l'en-tte Expditeur : Alt+1. Double frappe pour y amener le focus.\r"
		comm =comm + u"Lire l'en-tte Date : Alt+2.\r"
		comm =comm + u"Lire l'en-tte Destinataire : Alt+3. Double frappe pour y amener le focus.\r"
		comm =comm + u"Lire l'en-tte Copie  : Alt+4. Double frappe pour y amener le focus.\r"				
		comm =comm + u"Lire l'en-tte Copie cache  : Alt+5. Double frappe pour y amener le focus.\r"
		comm =comm + u"Lire l'en-tte Sujet : Alt+6. Double frappe pour y amener le focus.\r"
		comm =comm + u"Annonce des pices jointes : Alt+7. Double frappe pour y amener le focus.\r\r"
		comm =comm + u"	Autres commandes.\r\r"
		comm =comm + u"Aller  la liste des messages dans la fentre principale : Ctrl +L.\r"
		comm =comm + u"Lire le contenu du message dans le volet d'affichage : Ctrl +Windows +Flche bas.\r"	
		comm =comm + u"Aller  la zone de rdaction d'un message : Alt +8.\r"	
		comm =comm + u"Vrificateur orthographique, lire et peler le mot absent du dictionnaire : Alt+A.\r"
		comm =comm + u"Boite de dialogue pour la liste des raccourcis : NVDA+Control+H.\r"
		comm =comm + u"Pour fermer cette boite de dialogue : chapp, Entre ou Espace."
		
		liste.SetValue(comm)
		liste.SetFocus()
		taskSizer.Add(liste, wx.ALL|wx.EXPAND,wx.ALIGN_CENTRE, 5)
		mainSizer.Add(taskSizer,flag=wx.LEFT|wx.RIGHT|wx.BOTTOM|wx.ALIGN_CENTRE, border=5 )
		mainSizer.Fit(self)
		self.SetSizer(mainSizer)
		# the events
		self.Bind(wx.EVT_CLOSE, self.onClose)
		self.Bind(wx.EVT_CHAR_HOOK, self.onOutputChar) 
		
	def onClose(self, evt):
		self.Destroy()
		evt.Skip()	
			
	def onOutputChar(self, evt):
		key = evt.GetKeyCode()
		if key in (wx.WXK_RETURN,wx.WXK_NUMPAD_ENTER,wx.WXK_SPACE,wx.WXK_ESCAPE) :
			self.Close()
		evt.Skip()
				
	def run(cls):
		d = cls(None,wx.ID_ANY,"Commandes pour Windows Live Mail", True)
		d.CenterOnScreen()
		d.Show(True)
		PutWindowOnForeground(d.GetHandle(), 5, 0.2)
	run = classmethod(run)	